<?php
/**
 * @package GCal Events List
 * @version 2.0
 */
/*
Plugin Name: GCal Events List
Plugin URI: http://digitaladoptive.wordpress.com/gcal-events-list
Description: GCal Events List generates a list of events from a public Google Calendar. You need the calendar ID to make it work.
Author: veleno
Version: 2.0
Author URI: http://carlodaniele.it/
License: GPLv2 or later
Text Domain: gcal-events-list
Network:     true
*/

// Make sure we don't expose any info if called directly
if ( !function_exists( 'add_action' ) ) {
	echo 'Hi there!';
	exit;
}

function gcal_activate(){
	// Set the uninstallation hook
    register_uninstall_hook( __FILE__, 'gcal_uninstall' );
}
register_activation_hook( __FILE__, 'gcal_activate' );

function gcal_uninstall(){

	$plugin_data = get_plugin_data( __FILE__ );
    $option_name = 'widget_' . $plugin_data['Name'];;
 
	delete_option( $option_name );
	 
	// for site options in Multisite
	delete_site_option( $option_name );
}


class gcalEventsList extends WP_Widget {

	/**
	 * Set up the widget name and options
	 */
	public function __construct(){
	
		$widget_ops = array( 'description' => 'A widget that generates a list of events from a public Google Calendar' );
	
		parent::__construct( 'gcal-events-list', 'GCal Events List', $widget_ops );
	
	}

	/**
	 * Print the content of the widget on front-end
	 *
	 * @param array $args Widget arguments 
	 * @param array $instance 
	 *
	 */
	public function widget( $args, $instance ){

		echo $args['before_widget'];
		
		//widget title
		if ( ! empty( $instance['title'] ) ) {
			echo $args['before_title'] . apply_filters( 'widget_title', $instance['title'] ) . $args['after_title'];
		}	

		//google calendar parameters
		$params = array(
			'calendar' => ! empty( $instance['calendar'] ) ? $instance['calendar'] : '', //calendar ID
			'key' => ! empty( $instance['key'] ) ? $instance['key'] : '',
			'timeMin' => ( ! empty( $instance['timeMin'] ) && isDate( $instance['timeMin'] ) ) ? $instance['timeMin'] : date( 'Y-m-d' ),
			'maxResults' => ! empty( $instance['maxResults'] ) ? $instance['maxResults'] : '5',
			//'sortorder' => $instance['sortorder'],
			//'max-results' => $instance['maxresults'], //ricordati che il parametro del gcal  max-results
			//'start-min' => $instance['startmin'], //ricordati che il parametro del gcal  start-min
			//'start-max' => $instance['startmax'] //ricordati che il parametro del gcal  start-max	
		);
		
		if( ! empty( $params['calendar'] ) ){
			gcal_getData( $params );			
		}else{
			echo __( 'You shoud set the calendar ID to make this widget work' );
		}
		//close div
		echo $args['after_widget'];
	}
	
	/**
	 * Processing widget options on save
	 *
	 * @param array $new_instance The new options
	 * @param array $old_instance The previous options
	 *
	 * @return array $instance
	 */
	public function update( $new_instance, $old_instance ){
		
		$instance = array();
		
		$instance['title'] = ( ! empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';
		$instance['calendar'] = ( ! empty( $new_instance['calendar'] ) ) ? strip_tags( $new_instance['calendar'] ) : '';
		$instance['key'] = ( ! empty( $new_instance['key'] ) ) ? strip_tags( $new_instance['key'] ) : '';
		$instance['timeMin'] = ( ! empty( $new_instance['timeMin'] ) && isDate( $new_instance['timeMin'] ) ) ? strip_tags( $new_instance['timeMin'] ) : date( 'Y-m-d' );
		$instance['maxResults'] = ( ! empty( $new_instance['maxResults'] ) ) ? strip_tags( $new_instance['maxResults'] ) : '5';

		return $instance;
	}

	/**
	 * Outputs the options form on admin
	 *
	 * @param array $instance The widget options
	 *
	 */
	public function form( $instance ){

		$title = ! empty( $instance['title'] ) ? $instance['title'] : esc_html__( 'Title' );
		$calendar = ! empty( $instance['calendar'] ) ? $instance['calendar'] : '';
		$key = ! empty( $instance['key'] ) ? $instance['key'] : '';
		$timeMin = ( ! empty( $instance['timeMin'] ) && isDate( $instance['timeMin'] ) ) ? $instance['timeMin'] : date( 'Y-m-d' );
		$maxResults = ! empty( $instance['maxResults'] ) ? $instance['maxResults'] : '5';
		
		?>
		<p>
			<label for="<?php echo $this->get_field_id( 'title' ); ?>">Title: 
				<input class="widefat" 
					id="<?php echo $this->get_field_id( 'title' ); ?>"
					name="<?php echo $this->get_field_name( 'title' ); ?>"
					type="text"
					value="<?php echo $title; ?>" />
			</label>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'calendar' ); ?>">Calendar ID: 
				<input class="widefat" 
					id="<?php echo $this->get_field_id( 'calendar' ); ?>"
					name="<?php echo $this->get_field_name( 'calendar' ); ?>"
					type="text"
					value="<?php echo $calendar; ?>" />
			</label>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'key' ); ?>">Your API key: 
				<input class="widefat" 
					id="<?php echo $this->get_field_id( 'key' ); ?>"
					name="<?php echo $this->get_field_name( 'key' ); ?>"
					type="text"
					value="<?php echo $key; ?>" />
			</label>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('timeMin'); ?>">Start from (YYYY-MM-DD): 
				<input class="widefat" 
					id="<?php echo $this->get_field_id('timeMin'); ?>"
					name="<?php echo $this->get_field_name('timeMin'); ?>"
					type="text"
					value="<?php echo $timeMin; ?>" />
			</label>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('maxResults'); ?>"><?php echo __('Max results'); ?>: </label>		
			<select id="<?php echo $this->get_field_id( 'maxResults' ); ?>" name="<?php echo $this->get_field_name( 'maxResults' ); ?>" class="widefat">
				<option <?php if ( $maxResults == '1' ) echo 'selected="selected"'; ?>>1</option>
				<option <?php if ( $maxResults == '3' ) echo 'selected="selected"'; ?>>3</option>
				<option <?php if ( $maxResults == '5' ) echo 'selected="selected"'; ?>>5</option>
				<option <?php if ( $maxResults == '10' ) echo 'selected="selected"'; ?>>10</option>
				<option <?php if ( $maxResults == '15' ) echo 'selected="selected"'; ?>>15</option>
				<option <?php if ( $maxResults == '20' ) echo 'selected="selected"'; ?>>20</option>
			</select>
		</p>
		<?php
	}
}
add_action( 'widgets_init', function(){
	register_widget( 'gcalEventsList' );
});


/**
 * Retrieve data from a public calendar
 *
 * @param array $params An array of arguments
 *
 */
function gcal_getData( $params ){
	
	$calendarID = urlencode( $params['calendar'] );

	$key = $params['key'];
	$maxResults = ! empty( $params['maxResults'] ) ? ( "&maxResults=" . $params['maxResults'] ) : '5';

	if( ! empty( $params['timeMin'] ) && isDate( $params['timeMin'] ) ){

		$RFC3339 = $params['timeMin'] . 'T00:00:00Z';
		
		$timeMin = '&timeMin=' . $RFC3339;
	
	}else{
		
		$timeMin = '&timeMin=' . date( 'Y-m-d' ) . 'T00:00:00Z';
	
	}

	$resource = "https://www.googleapis.com/calendar/v3/calendars/" . $calendarID . "/events?key=" . $key . 
	$maxResults . "&orderBy=startTime" . "&singleEvents=true" . $timeMin;

	$request = wp_remote_get( $resource );

	if( is_wp_error( $request ) ) {
		return false;
	}

	$body = wp_remote_retrieve_body( $request );
	$data = json_decode( $body );

	if( ! empty( $data->items ) ) {
	?>
		<ul>
		<?php foreach( $data->items as $item ) { ?>
			<li><?php echo $item->summary; ?><br />
				<?php echo $item->start->date; ?>
			</li>
		<?php } ?>
		</ul>
	<?php
	}else{
		echo __( 'No items found!' );
	}
}

/**
 * Verify if is a valide date in Y-m-d format
 *
 * @param string $date 
 *
 * @return bool 
 */
function isDate( $date ){

	$dateTime = new DateTime();
    
    $d = $dateTime->createFromFormat( 'Y-m-d', $date );
    
    return $d && $d->format( 'Y-m-d' ) === $date;
}